/*
 *  JVending
 *  Copyright (C) 2006  Shane Isbell
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


package org.jvending.provisioning.impl;

import org.jvending.provisioning.stocking.handler.DescriptorHandler;
import org.jvending.provisioning.stocking.handler.StockingHandlerConfig;
import org.jvending.provisioning.stocking.StockingComponent;
import org.jvending.provisioning.stocking.StockingException;
import org.jvending.provisioning.model.clientbundle.ClientBundle;

import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.*;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ByteArrayInputStream;
import java.sql.Blob;

import org.jvending.provisioning.stocking.par.ClientBundleType;
import org.jvending.provisioning.stocking.par.VendorInfoType;
import org.jvending.provisioning.stocking.par.VendorNameType;
import org.jvending.provisioning.stocking.par.VendorUrlType;
import org.jvending.provisioning.stocking.par.VendorDescriptionType;
import org.jvending.provisioning.stocking.par.PreviewType;
import org.jvending.provisioning.stocking.par.IconType;
import org.jvending.provisioning.stocking.par.DisplayNameType;
import org.jvending.provisioning.stocking.par.DescriptionType;
import org.jvending.provisioning.stocking.par.ToolDescriptionsType;
import org.jvending.provisioning.stocking.par.UserDescriptionsType;
import org.jvending.provisioning.stocking.par.CopyrightType;
import org.jvending.provisioning.stocking.par.CatalogPropertyType;
import org.jvending.provisioning.stocking.par.DeviceRequirementType;
import org.jvending.provisioning.stocking.par.ContentFileType;
import org.jvending.provisioning.stocking.par.DescriptorFileType;

import org.jvending.provisioning.model.clientbundle.VendorInfo;
import org.jvending.provisioning.model.clientbundle.Preview;
import org.jvending.provisioning.model.clientbundle.DisplayName;
import org.jvending.provisioning.model.clientbundle.ToolDescription;
import org.jvending.provisioning.model.clientbundle.Description;
import org.jvending.provisioning.model.clientbundle.Icon;
import org.jvending.provisioning.model.clientbundle.UserDescription;
import org.jvending.provisioning.model.clientbundle.Copyright;
import org.jvending.provisioning.model.clientbundle.CatalogProperty;
import org.jvending.provisioning.model.clientbundle.DeviceRequirement;
import org.jvending.provisioning.model.clientbundle.ContentFile;
import org.jvending.provisioning.model.clientbundle.DescriptorFile;
import org.jvending.provisioning.config.MimeTypeRepository;

import org.hibernate.Hibernate;

import javax.provisioning.ProvisioningException;

/**
 * Translates a JAXB generated client bundle into Hibernate generated JavaBeans.
 * @author Shane Isbell
 * @since 2.0.0
 */

final class ClientBundleTranslator {

    /**
     * Logger instance
     */
    private static Logger logger = Logger.getLogger("ClientBundleTranslator");

    static ClientBundle translate(ClientBundleType bundleType,
                                  Map contentMap,
                                  StockingHandlerConfig stockingHandlerConfig,
                                  MimeTypeRepository mimeTypeRepository) throws ProvisioningException {
        if(stockingHandlerConfig == null) throw new ProvisioningException("StockingHandlerConfig is null");
        if(bundleType == null) throw new ProvisioningException("ClientBundleType is null");

        ClientBundle bundle = new ClientBundle();
        bundle.setContentId(bundleType.getContentId());
        bundle.setBundleType(bundleType.getBundleType());
        bundle.setVersion(bundleType.getVersion());
        bundle.setUploadTime(new Date(System.currentTimeMillis()));
        bundle.setVendorInfo(translateVendorInfo(bundleType.getVendorInfo()));
        bundle.setPreviews(translatePreviews(bundleType.getPreview(), contentMap));
        bundle.setToolDescription(translateToolDescriptions(bundleType.getToolDescriptions(), contentMap));
        bundle.setUserDescription(translateUserDescriptions(bundleType.getUserDescriptions(), contentMap));
        bundle.setCopyrights(translateCopyrights(bundleType.getCopyright(), contentMap));
        bundle.setCatalogProperties(translateCatalogProperties(bundleType.getCatalogProperty()));
        bundle.setDeviceRequirements(translateDeviceRequirements(bundleType.getDeviceRequirement()));

        DescriptorFileType descriptorFileType = bundleType.getDescriptorFile();
        if (descriptorFileType != null) {
            DescriptorFile descriptorFile = translateDescriptorFile(bundleType.getDescriptorFile(), contentMap);
            List contentFiles = translateContentFilesFromDescriptor(descriptorFileType, contentMap,
                    stockingHandlerConfig, mimeTypeRepository);
            List descriptorFiles = new ArrayList();
            descriptorFiles.add(descriptorFile);
            String hashId = createHashId(aggregateDescriptorsToBytes(descriptorFiles),
                    aggregateContentToBytes(contentFiles));
            bundle.setHashId(hashId);
            bundle.setDescriptorFile(descriptorFile);
            bundle.setContentFilesFromDescriptor(contentFiles);
        } else {
            ContentFileType contentFileType = bundleType.getContentFile();
            if (contentFileType == null) {
                logger.info("Both descriptor file and content file are null. Content Id = " +
                        bundleType.getContentId());
            }
            ContentFile contentFile = translateContentFile(contentFileType, contentMap);
            List contentFiles = new ArrayList();
            contentFiles.add(contentFile);
            String hashId = createHashId(null,
                    aggregateContentToBytes(contentFiles));

            bundle.setContentFile(contentFile);
            bundle.setHashId(hashId);
        }

        return bundle;
    }

    /**
     * Creates a hash (SHA1) for the content. If there is a descriptor and content associated with that descriptor,
     * the returned ID includes a hash of both. Otherwise it just includes a hash ID of either a descriptor or a
     * content.
     * @param descriptor
     * @param content
     * @return hash ID
     */
    private static String createHashId(byte[] descriptor, byte[] content) {
        if (descriptor == null && content == null)
            return null;

        int descriptorLength = (descriptor != null) ? descriptor.length : 0;
        int contentLength = (content != null) ? content.length : 0;

        byte[] b = new byte[(descriptorLength + contentLength)];

        if (descriptor != null)
            System.arraycopy(descriptor, 0, b, 0, descriptorLength);

        if (content != null && descriptor != null)
            System.arraycopy(content, 0, b, descriptorLength, contentLength);
        else if (content != null && descriptor == null)
            System.arraycopy(content, 0, b, 0, descriptorLength);

        try {
            String bundleId = new String(MessageDigest.getInstance("SHA1").digest(b));
            logger.finest("Created bundle id: ID = " + bundleId);
            return bundleId;
        } catch (NoSuchAlgorithmException e) {
            logger.log(Level.INFO, "Could not create bundle id", e);
            return null;
        }
    }

    /**
     * Aggregates a list of content files into a single byte array.
     * @param contentFiles
     * @return array of aggregated content
     */
    private static byte[] aggregateContentToBytes(List contentFiles) {
        ByteArrayOutputStream aggregatedContent = new ByteArrayOutputStream();
        if (contentFiles == null) {
            logger.warning("Could not aggregate content bytes");
            return aggregatedContent.toByteArray();
        }
        for (Iterator i = contentFiles.iterator(); i.hasNext();) {
            ContentFile contentFile = (ContentFile) i.next();
            if (contentFile.getBytes() == null) return aggregatedContent.toByteArray();
            try {
                aggregatedContent.write(contentFile.getBytes());
            } catch (IOException e) {
                logger.log(Level.WARNING, "Could not aggregate content bytes", e);
            }
        }
        return aggregatedContent.toByteArray();
    }

    /**
     * Aggregates a list of descriptors files into a single byte array.
     * @param contentFiles
     * @return array of aggregated descriptors
     */
    private static byte[] aggregateDescriptorsToBytes(List contentFiles) {
        ByteArrayOutputStream aggregatedContent = new ByteArrayOutputStream();
        for (Iterator i = contentFiles.iterator(); i.hasNext();) {
            DescriptorFile contentFile = (DescriptorFile) i.next();
            byte[] b = contentFile.getBytes();
            if (b == null) return aggregatedContent.toByteArray();
            try {
                aggregatedContent.write(b);
            } catch (IOException e) {
                logger.log(Level.WARNING, "Could not aggregate descriptor bytes", e);
            }
        }
        return aggregatedContent.toByteArray();
    }



    static private List translateContentFilesFromDescriptor(DescriptorFileType descriptorFileType, Map contentMap,
                                                            StockingHandlerConfig stockingHandlerConfig,
                                                            MimeTypeRepository mimeTypeRepository)
            throws ProvisioningException{
        StockingComponent stockingComponent = stockingHandlerConfig.getStockingContext().getStockingComponent();
        String handlerName = stockingHandlerConfig.getStockingHandlerName();

        String mimeType = descriptorFileType.getMimeType();
        if (mimeType == null || mimeType.trim().equals(""))
            mimeType = mimeTypeRepository.getMimeTypeFromUri(descriptorFileType.getValue());

        DescriptorHandler descriptorHandler = null;
        try {
            descriptorHandler = stockingComponent.createDescriptorHandler(handlerName, mimeType);
            if(descriptorHandler == null)
                throw new ProvisioningException("Could not create DescriptorHandler: Handler = " + handlerName
                        + ", Mime-Type = " + mimeType);
            return descriptorHandler.getContentFiles(descriptorFileType, contentMap);
        } catch (StockingException e) {
            e.printStackTrace();
            throw new ProvisioningException("Could not create DescriptorHandler: Message = " + e.getMessage());
        }

    }

    static private DescriptorFile translateDescriptorFile(DescriptorFileType descriptorFileType, Map contentMap) {
        if (descriptorFileType == null) return null;
        String fileUri = descriptorFileType.getValue();
        String mimeType = descriptorFileType.getMimeType();
        if (isRemoteUri(fileUri)) {
            DescriptorFile descriptorFile = new DescriptorFile();
            descriptorFile.setContent(null);
            descriptorFile.setFileUri(fileUri);
            descriptorFile.setMimeType(mimeType);
            return descriptorFile;
        } else {
            byte[] content = (byte[]) contentMap.get(fileUri);
            Blob contentBlob = null;
            if (content != null) {
                contentBlob = Hibernate.createBlob(new ByteArrayInputStream(content), content.length);
            } else {
                contentBlob = Hibernate.createBlob(new ByteArrayInputStream(new byte[0]), 0);
            }
            DescriptorFile descriptorFile = new DescriptorFile();
            descriptorFile.setContent(contentBlob);
            descriptorFile.setFileUri(fileUri);
            descriptorFile.setMimeType(mimeType);

            descriptorFile.setBytes(content);
            return descriptorFile;
        }

    }

    private static boolean isRemoteUri(String uri) {
        return (uri != null && (uri.startsWith("http://") || uri.startsWith("ftp://") || uri.startsWith("https://")));
    }


    static private ContentFile translateContentFile(ContentFileType contentFileType, Map contentMap) {
        if (contentFileType == null) return null;
        String fileUri = contentFileType.getValue();
        String mimeType = contentFileType.getMimeType();
        byte[] content = (byte[]) contentMap.get(fileUri);
        Blob contentBlob = null;
        if (content != null) {
            contentBlob = Hibernate.createBlob(new ByteArrayInputStream(content), content.length);
        } else {
            contentBlob = Hibernate.createBlob(new ByteArrayInputStream(new byte[0]), 0);
        }

        logger.finest("Content File: URI = " + fileUri + ", Mime-Type = " + mimeType);
        ContentFile contentFile = new ContentFile();
        contentFile.setContent(contentBlob);
        contentFile.setFileUri(fileUri);
        contentFile.setMimeType(mimeType);
        contentFile.setBytes(content);
        return contentFile;
    }

    static private List translateDeviceRequirements(List deviceRequirements) {
        if (deviceRequirements == null) return null;
        List list = new ArrayList();

        for (Iterator i = deviceRequirements.iterator(); i.hasNext();) {
            DeviceRequirementType deviceRequirementType = (DeviceRequirementType) i.next();
            String value = deviceRequirementType.getRequirementValue();
            String name = deviceRequirementType.getRequirementName();
            DeviceRequirement requirement = new DeviceRequirement();
            requirement.setName(name);
            requirement.setValue(value);
            logger.finest("Device Requirement: Name = " + name + ", Value = " + value);
            list.add(requirement);
        }
        return list;
    }

    static private List translateCatalogProperties(List catalogProperties) {
        if (catalogProperties == null) return null;
        List list = new ArrayList();

        for (Iterator i = catalogProperties.iterator(); i.hasNext();) {
            CatalogPropertyType catalogPropertyType = (CatalogPropertyType) i.next();
            String value = catalogPropertyType.getPropertyValue();
            String name = catalogPropertyType.getPropertyName();
            CatalogProperty property = new CatalogProperty();
            property.setName(name);
            property.setValue(value);
            logger.finest("Property: Name = " + name + ", Value = " + value);
            list.add(property);
        }
        return list;
    }

    static private List translateCopyrights(List copyrights, Map contentMap) {
        if (copyrights == null) return null;
        List list = new ArrayList();

        for (Iterator i = copyrights.iterator(); i.hasNext();) {
            CopyrightType copyrightType = (CopyrightType) i.next();
            String value = copyrightType.getValue();
            String uri = copyrightType.getUri();
            if (uri != null) {
                byte[] copyrightValue = (byte[]) contentMap.get(uri);
                if (copyrightValue != null) value = new String(copyrightValue);
            }
            String locale = copyrightType.getLocale();
            Copyright copyright = new Copyright();
            copyright.setCopyright(value);
            copyright.setLocale(locale);
            copyright.setUri(uri);
            logger.finest("Copyright: Copyright = " + value + ", Locale = " + locale + ", URI = " + uri);
            list.add(copyright);
        }
        return list;
    }

    static private UserDescription translateUserDescriptions(UserDescriptionsType userDescriptionsType, Map contentMap) {
        if (userDescriptionsType == null) return null;
        UserDescription userDescription = new UserDescription();
        userDescription.setDescriptions(translateDescriptions(userDescriptionsType.getDescription()));
        userDescription.setDisplayNames(translateDisplayNames(userDescriptionsType.getDisplayName()));
        userDescription.setIcons(translateIcons(userDescriptionsType.getIcon(), contentMap));
        return userDescription;
    }

    static private ToolDescription translateToolDescriptions(ToolDescriptionsType toolDescriptionsType, Map contentMap) {
        if (toolDescriptionsType == null) return null;
        ToolDescription toolDescription = new ToolDescription();
        toolDescription.setDescriptions(translateDescriptions(toolDescriptionsType.getDescription()));
        toolDescription.setDisplayNames(translateDisplayNames(toolDescriptionsType.getDisplayName()));
        toolDescription.setIcons(translateIcons(toolDescriptionsType.getIcon(), contentMap));
        return toolDescription;

    }

    static private List translateDescriptions(List descriptions) {
        if (descriptions == null) return null;
        List list = new ArrayList();

        for (Iterator i = descriptions.iterator(); i.hasNext();) {
            DescriptionType descriptionType = (DescriptionType) i.next();
            String value = descriptionType.getValue();
            String locale = descriptionType.getLocale();
            Description description = new Description();
            description.setDescription(value);
            description.setLocale(locale);
            logger.finest("Description: File Name = " + value + ", Locale = " + locale);
            list.add(description);
        }
        return list;
    }


    static private List translateDisplayNames(List displayNames) {
        if (displayNames == null) return null;
        List list = new ArrayList();

        for (Iterator i = displayNames.iterator(); i.hasNext();) {
            DisplayNameType displayNameType = (DisplayNameType) i.next();
            String value = displayNameType.getValue();
            String locale = displayNameType.getLocale();
            DisplayName displayName = new DisplayName();
            displayName.setDisplayName(value);
            displayName.setLocale(locale);
            logger.finest("Display Name: File Name = " + value + ", Locale = " + locale);
            list.add(displayName);
        }
        return list;
    }

    static private List translateIcons(List iconTypes, Map contentMap) {
        if (iconTypes == null || contentMap == null) return null;
        List icons = new ArrayList();

        for (Iterator i = iconTypes.iterator(); i.hasNext();) {
            IconType iconType = (IconType) i.next();
            String fileUri = iconType.getValue();
            String mimeType = iconType.getMimeType();
            String locale = iconType.getLocale();
            byte[] content = (byte[]) contentMap.get(fileUri);
            Blob contentBlob = null;
            if (content != null) {
                contentBlob = Hibernate.createBlob(new ByteArrayInputStream(content), content.length);
            } else {
                contentBlob = Hibernate.createBlob(new ByteArrayInputStream(new byte[0]), 0);
            }

            logger.finest("Icon: File Name = " + fileUri + ", Mime-Type = " + mimeType +
                    ", Locale = " + locale);
            Icon icon = new Icon();
            icon.setFile(contentBlob);
            icon.setFileUri(fileUri);
            icon.setLocale(locale);
            icon.setMimeType(mimeType);
            icons.add(icon);
        }
        return icons;
    }


    static private List translatePreviews(List previewTypes, Map contentMap) {
        if (previewTypes == null || contentMap == null) return null;
        List previews = new ArrayList();

        for (Iterator i = previewTypes.iterator(); i.hasNext();) {
            PreviewType previewType = (PreviewType) i.next();
            String fileUri = previewType.getValue();
            String mimeType = previewType.getMimeType();
            String locale = previewType.getLocale();
            byte[] content = (byte[]) contentMap.get(fileUri);
            Blob contentBlob = null;
            if (content != null) {
                contentBlob = Hibernate.createBlob(new ByteArrayInputStream(content), content.length);
            } else {
                contentBlob = Hibernate.createBlob(new ByteArrayInputStream(new byte[0]), 0);
            }

            logger.finest("Preview: File Name = " + fileUri + ", Mime-Type = " + mimeType +
                    ", Locale = " + locale);
            Preview preview = new Preview();
            preview.setFile(contentBlob);
            preview.setFileUri(fileUri);
            preview.setLocale(locale);
            preview.setMimeType(mimeType);
            previews.add(preview);
        }
        return previews;
    }

    static private VendorInfo translateVendorInfo(VendorInfoType vendorInfoType) {
        if (vendorInfoType == null) return null;
        VendorInfo vendorInfo = new VendorInfo();
        VendorNameType vendorNameType = vendorInfoType.getVendorName();
        if (vendorNameType != null) {
            vendorInfo.setVendorName(vendorNameType.getValue());
            vendorInfo.setVendorNameLocale(vendorNameType.getLocale());
        }

        VendorUrlType vendorUrlType = vendorInfoType.getVendorUrl();
        if (vendorUrlType != null) {
            vendorInfo.setVendorUrl(vendorUrlType.getValue());
            vendorInfo.setVendorUrlLocale(vendorUrlType.getLocale());
        }

        VendorDescriptionType vendorDescriptionType = vendorInfoType.getVendorDescription();
        if (vendorDescriptionType != null) {
            vendorInfo.setVendorDescription(vendorDescriptionType.getValue());
            vendorInfo.setVendorDescriptionLocale(vendorDescriptionType.getLocale());
        }

        return vendorInfo;
    }
}
